from checkov.terraform.checks.provider.aws import *  # noqa
from checkov.terraform.checks.provider.linode import *  # noqa
from checkov.terraform.checks.provider.bridgecrew import *  # noqa
from checkov.terraform.checks.provider.oci import *  # noqa
from checkov.terraform.checks.provider.openstack import *  # noqa
from checkov.terraform.checks.provider.panos import *  # noqa
